<?php

namespace Cas\Service\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cas\Service\Http\Requests\ServicesRequest;
use Cas\Service\Interfaces\ServicesRepositoryInterface;
use Cas\Service\Models\Services;

/**
 * Resource controller class for services.
 */
class ServicesResourceController extends BaseController
{

    /**
     * Initialize services resource controller.
     *
     * @param type ServicesRepositoryInterface $services
     *
     * @return null
     */
    public function __construct(ServicesRepositoryInterface $services)
    {
        parent::__construct();
        $this->repository = $services;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cas\Service\Repositories\Criteria\ServicesResourceCriteria::class);
    }

    /**
     * Display a list of services.
     *
     * @return Response
     */
    public function index(ServicesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cas\Service\Repositories\Presenter\ServicesPresenter::class)
                ->$function();
        }

        $services = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('service::services.names'))
            ->view('service::services.index', true)
            ->data(compact('services', 'view'))
            ->output();
    }

    /**
     * Display services.
     *
     * @param Request $request
     * @param Model   $services
     *
     * @return Response
     */
    public function show(ServicesRequest $request, Services $services)
    {

        if ($services->exists) {
            $view = 'service::services.show';
        } else {
            $view = 'service::services.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('service::services.name'))
            ->data(compact('services'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new services.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ServicesRequest $request)
    {

        $services = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('service::services.name')) 
            ->view('service::services.create', true) 
            ->data(compact('services'))
            ->output();
    }

    /**
     * Create new services.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ServicesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $services                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('service::services.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('service/services/' . $services->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/service/services'))
                ->redirect();
        }

    }

    /**
     * Show services for editing.
     *
     * @param Request $request
     * @param Model   $services
     *
     * @return Response
     */
    public function edit(ServicesRequest $request, Services $services)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('service::services.name'))
            ->view('service::services.edit', true)
            ->data(compact('services'))
            ->output();
    }

    /**
     * Update the services.
     *
     * @param Request $request
     * @param Model   $services
     *
     * @return Response
     */
    public function update(ServicesRequest $request, Services $services)
    {
        try {
            $attributes = $request->all();

            $services->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('service::services.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('service/services/' . $services->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('service/services/' . $services->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the services.
     *
     * @param Model   $services
     *
     * @return Response
     */
    public function destroy(ServicesRequest $request, Services $services)
    {
        try {

            $services->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('service::services.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('service/services/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('service/services/' . $services->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple services.
     *
     * @param Model   $services
     *
     * @return Response
     */
    public function delete(ServicesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('service::services.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('service/services'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/service/services'))
                ->redirect();
        }

    }

    /**
     * Restore deleted services.
     *
     * @param Model   $services
     *
     * @return Response
     */
    public function restore(ServicesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('service::services.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/service/services'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/service/services/'))
                ->redirect();
        }

    }

}
